
// All string handling functions here in one place.

unit StringH;

interface

uses SysUtils, Classes;


// General string handling functions:
function AddString(const sOriginal: String; const sFull: String): String;
function sCase(const sString: String; const bLower: Boolean): String;
function IsValidFileName(const sFileName: String): Boolean;
function RemoveFileExt(const sFileName: String): String;
function OnlyNumeric(const Key: Char): Char;
function NonLetter(const sChar: String): Boolean;

// Functions for Rename 'Em!:
function fReplace(const sFileName: String; const sFind: String; const sNew: String; const bCaseSensitive: Boolean): String;
function fAdd(const sFileName: String; const iPosition: Integer; const sNew: String): String;
function fAutoNumber(const sNumber: String; const sBefore: String; const sAfter: String): String;
function fChangeCase(const sFileName: String; const iCaseType: Integer): String;
function fTrim(const sFileName: String; const iAmount: Integer; const bStart: Boolean; const bTotalLen: Boolean): String;
function fExtension(const sFileName: String; const sNewExtension: String): String;
function fAdvancedAdd(const sFileName: String; const iA: Integer; const iB: Integer; const sNew: String; const bBefore: Boolean): String;
function fFileNamesFromID3(const sFileName: String; const sFormat: String): String;
//function fFileNamesToID3(sFullFileName: String; sSeparator: String; sFileNameOnly: String): String;


type
  TID3Rec = packed record
            Tag: array[0..2] of Char;
            Title,
            Artist,
            Album,
            Year: array[0..29] of Char;
            Comment: array[0..3] of Char;
            Genre: Byte;
  end;


const
        ENTER = #13 + #10;
        MaxID3Genre = 147;
        ID3Genre: array[0..MaxID3Genre] of string = (
        'Blues', 'Classic Rock', 'Country', 'Dance', 'Disco', 'Funk', 'Grunge',
        'Hip-Hop', 'Jazz', 'Metal', 'New Age', 'Oldies', 'Other', 'Pop', 'R&B',
        'Rap', 'Reggae', 'Rock', 'Techno', 'Industrial', 'Alternative', 'Ska',
        'Death Metal', 'Pranks', 'Soundtrack', 'Euro-Techno', 'Ambient',
        'Trip-Hop', 'Vocal', 'Jazz+Funk', 'Fusion', 'Trance', 'Classical',
        'Instrumental', 'Acid', 'House', 'Game', 'Sound Clip', 'Gospel',
        'Noise', 'AlternRock', 'Bass', 'Soul', 'Punk', 'Space', 'Meditative',
        'Instrumental Pop', 'Instrumental Rock', 'Ethnic', 'Gothic',
        'Darkwave', 'Techno-Industrial', 'Electronic', 'Pop-Folk',
        'Eurodance', 'Dream', 'Southern Rock', 'Comedy', 'Cult', 'Gangsta',
        'Top 40', 'Christian Rap', 'Pop/Funk', 'Jungle', 'Native American',
        'Cabaret', 'New Wave', 'Psychadelic', 'Rave', 'Showtunes', 'Trailer',
        'Lo-Fi', 'Tribal', 'Acid Punk', 'Acid Jazz', 'Polka', 'Retro',
        'Musical', 'Rock & Roll', 'Hard Rock', 'Folk', 'Folk-Rock',
        'National Folk', 'Swing', 'Fast Fusion', 'Bebob', 'Latin', 'Revival',
        'Celtic', 'Bluegrass', 'Avantgarde', 'Gothic Rock', 'Progressive Rock',
        'Psychedelic Rock', 'Symphonic Rock', 'Slow Rock', 'Big Band',
        'Chorus', 'Easy Listening', 'Acoustic', 'Humour', 'Speech', 'Chanson',
        'Opera', 'Chamber Music', 'Sonata', 'Symphony', 'Booty Bass', 'Primus',
        'Porn Groove', 'Satire', 'Slow Jam', 'Club', 'Tango', 'Samba',
        'Folklore', 'Ballad', 'Power Ballad', 'Rhythmic Soul', 'Freestyle',
        'Duet', 'Punk Rock', 'Drum Solo', 'Acapella', 'Euro-House', 'Dance Hall',
        'Goa', 'Drum & Bass', 'Club-House', 'Hardcore', 'Terror', 'Indie',
        'BritPop', 'Negerpunk', 'Polsk Punk', 'Beat', 'Christian Gangsta Rap',
        'Heavy Metal', 'Black Metal', 'Crossover', 'Contemporary Christian',
        'Christian Rock', 'Merengue', 'Salsa', 'Trash Metal', 'Anime', 'Jpop',
        'Synthpop'  {and probably more to come}
        );

implementation

function AddString(const sOriginal: String; const sFull: String): String;
var
        sDone: String;

begin
// Add zeros in front of a number, until it's as long in length as the full number.
// Eg. IMAGE001, when the last image is IMAGE100. Used for auto-numering.
sDone := sOriginal;
if (Length(sOriginal) >= Length(sFull)) then begin
                                             Result := sOriginal;
                                             exit;
                                             end;

sDone := '0' + sDone;
if (Length(sDone) < Length(sFull)) then sDone := AddString(sDone, sFull);
Result := sDone;
end;

function RemoveFileExt(const sFileName: String): String;
var
        iLen: Integer;
        oldExtension: String;

begin
// Removes file's extension and returns the filename without it as a result.
iLen := Length(sFileName);
oldExtension := ExtractFileExt(sFileName);

if (Length(oldExtension) <= 0) then begin
                                    Result := sFileName;
                                    end
                                    else begin
                                    Result := copy(sFileName, 1, iLen - Length(oldExtension));
                                    end;
end;

function sCase(const sString: String; const bLower: Boolean): String;
var
        i: Integer;
        sDone: String;
        sChar: String;

begin
// Delphi/Kylix has a bug: it doesn't convert scandinavian letters, like
// ,  and  (also some else) to Upper/LowerCase with the standard built-in-
// functions. So I needed to make one by myself.
sDone := '';
for i := 1 to Length(sString) do begin
         sChar := copy(sString, i, 1);
         if (bLower) then begin
                          sChar := LowerCase(sChar);
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          end
                          else begin
                          sChar := UpperCase(sChar);
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          if (sChar = '') then sChar := '';
                          end;
         sDone := sDone + sChar;
end;

Result := sDone;
end;

function IsValidFileName(const sFileName: String): Boolean;
// Does filename start or end with space? Does it contain invalid characters?
// Checks the filename validity.
begin
Result := False;

if (Length(sFileName) <= 0) then exit;
if (copy(sFileName, 1, 1) = ' ') then exit;
if (copy(sFileName, Length(sFileName) - 1, 1) = ' ') then exit;

if (Pos('\', sFileName) > 0) then exit;
if (Pos('/', sFileName) > 0) then exit;
if (Pos('"', sFileName) > 0) then exit;
if (Pos(':', sFileName) > 0) then exit;
if (Pos('?', sFileName) > 0) then exit;
if (Pos('*', sFileName) > 0) then exit;
if (Pos('|', sFileName) > 0) then exit;
if (Pos('<', sFileName) > 0) then exit;
if (Pos('>', sFileName) > 0) then exit;

Result := True;
end;

function OnlyNumeric(const Key: Char): Char;
// Only accept numbers to textboxes.
begin
if (Ord(Key) = 8) then begin
                  OnlyNumeric := Key;
                  exit;
                  end;

if (Ord(Key) < 48) or (Ord(Key) > 57) then begin
                   OnlyNumeric := Char(0);
                   exit;
                   end
                   else begin
                   OnlyNumeric := Key;
                   end;
end;

function NonLetter(const sChar: String): Boolean;
var
        iCharCode: Integer;
        bSpecial: Boolean;

begin
// Is a char letter (A, B, etc.) or a symbol/mark (-, /, \, etc.)?
// Needed for fChangeCase, to see which letters are converted to which case.
if (Length(sChar) < 1) then begin
                            Result := True;
                            exit;
                            end;

iCharCode := Ord(sChar[1]);
bSpecial := False;
if (sChar = '''') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;
if (sChar = '') then bSpecial := True;

if (bSpecial) then begin
                   Result := False;
                   exit;
                   end;
if (iCharCode < 65) then begin
                         Result := True;
                         exit;
                         end;
if (iCharCode > 90) and (iCharCode < 97) then begin
                                              Result := True;
                                              exit;
                                              end;
if (iCharCode > 122) then begin
                          Result := True;
                          exit;
                          end;

Result := False;
end;

function fReplace(const sFileName: String; const sFind: String; const sNew: String; const bCaseSensitive: Boolean): String;
var
        sDone: String;
        curChar: String;
        sFileNameC: String;
        i: Integer;
        iLen: Integer;
	nLen: Integer;
        sString: String;
        sFindC: String;

begin
// Replace string from filenames case sensitive, or non-case sensitive.
sDone := '';
if not (bCaseSensitive) then sFileNameC := LowerCase(sFileName) else sFileNameC := sFileName;
if not (bCaseSensitive) then sFindC := LowerCase(sFind) else sFindC := sFind;
iLen := Length(sFind);
nLen := Length(sNew);

for i := 1 to Length(sFileName) do begin
	 curChar := copy(sFileName, i, 1);

         sString := copy(sFileNameC, i, iLen);
         if not (bCaseSensitive) then sString := LowerCase(sString);
         if (sString = sFindC) then begin
				    sDone := sDone + sNew + copy(sFileName, i + iLen, Length(sFileName) - i - iLen + 1);
                                    if (Pos(sFind, sNew) <= 0) then begin
				                                   Result := fReplace(sDone, sFind, sNew, bCaseSensitive); // The function calls itself as long as all occurrences of the string have been replaced.
								   end
								   else begin
                                                                   Result := sDone;
								   end;
                                    exit;
                                    end;

         sDone := sDone + curChar;
end;



Result := sDone;
end;

function fAdd(const sFileName: String; const iPosition: Integer; const sNew: String): String;
var
        sLen: Integer;
        sLeft: String;
        sRight: String;
        i: Integer;

begin
// Add string to filename to specified position.
sLen := Length(sFileName);
if (iPosition > sLen) then begin
                           Result := sFileName + sNew;
                           exit;
                           end;
if (iPosition <= 1) then begin
                           Result := sNew + sFileName;
                           exit;
                           end;

for i := 1 to Length(sFileName) do begin
         if (i = iPosition) then begin
                                 sLeft := copy(sFileName, 1, i - 1);
                                 sRight := copy(sFileName, i, sLen - i + 1);
                                 Result := sLeft + sNew + sRight;
                                 exit;
                                 end;
end;

Result := sFileName;
end;

function fAutoNumber(const sNumber: String; const sBefore: String; const sAfter: String): String;
var
        sDone: String;

begin
// Autonumber files.
sDone := sBefore + sNumber + sAfter;
Result := sDone;
end;

function fChangeCase(const sFileName: String; const iCaseType: Integer): String;
var
        i: Integer;
        sDone: String;
        sChar: String;
        sLastChar: String;

begin
// Change the case from filenames.
sDone := '';
sLastChar := '';
if (iCaseType = 0) then begin // ALL UPPERCASE
                        Result := sCase(sFileName, False);
                        exit;
                        end;
if (iCaseType = 1) then begin // all lowercase
                        Result := sCase(sFileName, True);
                        exit;
                        end;

for i := 1 to Length(sFileName) do begin
         sChar := copy(sFileName, i, 1);

         if (iCaseType = 2) then begin // First Letter Uppercase
                                 if (NonLetter(sLastChar)) then sChar := sCase(sChar, False) else sChar := sCase(sChar, True);
                                 end;
         if (iCaseType = 3) then begin // fIRST lETTER lOWERCASE
                                 if (NonLetter(sLastChar)) then sChar := sCase(sChar, True) else sChar := sCase(sChar, False);
                                 end;
         if (iCaseType = 4) then begin // Change case (a -> A, A -> a)
                                 if (sCase(sChar, True) = sChar) then begin
                                                                      sChar := sCase(sChar, False);
                                                                      end
                                                                      else begin
                                                                      sChar := sCase(sChar, True);
                                                                      end;
                                 end;

         sDone := sDone + sChar;
         sLastChar := sChar;
end;

Result := sDone;
end;

function fTrim(const sFileName: String; const iAmount: Integer; const bStart: Boolean; const bTotalLen: Boolean): String;
var
        sDone: String;
        iNewLen: Integer;
        iLen: Integer;

begin
// Trim filenames from start or end.
iLen := Length(sFileName);
if (iAmount >= iLen) then begin
                          if (bTotalLen) then begin
                                              Result := sFileName;
                                              exit;
                                              end
                                              else begin
                                              Result := '';
                                              exit;
                                              end;
                          end;

if (bTotalLen) then begin // Make filenames this (iAmount) long
                    iNewLen := iLen - iAmount;
                    if (bStart) then begin
                                     sDone := copy(sFileName, iNewLen + 1, iLen - iNewLen + 1);
                                     end
                                     else begin
                                     sDone := copy(sFileName, 1, iLen - iNewLen);
                                     end;
                    end
                    else begin // Trim the selected amount
                    if (bStart) then begin
                                     sDone := copy(sFileName, iAmount + 1, iLen - iAmount + 1);
                                     end
                                     else begin
                                     sDone := copy(sFileName, 1, iLen - iAmount);
                                     end;
                    end;

Result := sDone;
end;


function fExtension(const sFileName: String; const sNewExtension: String): String;
var
        sDone: String;

begin
// Switch extension from filename.
sDone := RemoveFileExt(sFileName);

if (Length(sNewExtension) > 0) then Result := sDone + '.' + sNewExtension else Result := sDone;
end;

function fAdvancedAdd(const sFileName: String; const iA: Integer; const iB: Integer; const sNew: String; const bBefore: Boolean): String;
var
        sLen: Integer;
        sDone: String;
        iCharCode: Integer;
        curChar: String;
        i: Integer;

begin
// Add strings based on ASCII-codes (Advanced Add).
sLen := Length(sFileName);
sDone := '';

for i := 1 to sLen do begin
         curChar := sFileName[i];
         iCharCode := Ord(sFileName[i]);
         if (iCharCode >= iA) and (iCharCode <= iB) and (i > 1) then begin
                                                         if (bBefore) then begin
                                                                           sDone := sDone + sNew + curChar;
                                                                           end
                                                                           else begin
                                                                           sDone := sDone + curChar + sNew;
                                                                           end;
                                                         end
                                                         else begin
                                                         sDone := sDone + curChar;
                                                         end;
end;

Result := sDone;
end;









function fFileNamesFromID3(const sFileName: String; const sFormat: String): String;
var
    ID3: TID3Rec;
    fMP3: TFileStream;
    sTitle: String;
    sArtist: String;
    sAlbum: String;
    sYear: String;
    sGenre: String;
    sComment: String;
    sNewFileName: String;
    
begin
fMP3 := TFileStream.Create(sFileName, fmOpenRead);

try
        fMP3.position := fMP3.size - 128;
        fMP3.Read(ID3, SizeOf(ID3));
finally
        fMP3.Free;
end;

if (ID3.Tag <> 'TAG') then begin
                           Result := '';
                           exit;
                           end
                           else begin
                           sTitle := ID3.Title;
                           sArtist :=ID3.Artist;
                           sAlbum :=ID3.Album;
                           sYear := ID3.Year;
                           if ID3.Genre in [0..MaxID3Genre] then sGenre := ID3Genre[ID3.Genre] else sGenre := IntToStr(ID3.Genre);
                           sComment := ID3.Comment;
                           end;

sNewFileName := sFormat;
sNewFileName := fReplace(sNewFileName, '%artist%', sArtist, False);
sNewFileName := fReplace(sNewFileName, '%title%', sTitle, False);
sNewFileName := fReplace(sNewFileName, '%album%', sAlbum, False);
sNewFileName := fReplace(sNewFileName, '%year%', sYear, False);
sNewFileName := fReplace(sNewFileName, '%genre%', sGenre, False);
sNewFileName := fReplace(sNewFileName, '%comment%', sComment, False);

Result := sNewFileName;
end;



// The following function might be enabled in the future.
// It allows to parse ID3-tags out of filenames form Artist - Title.
{
function fFileNamesToID3(sFullFileName: String; sSeparator: String; sFileNameOnly: String): String;
var
        fMP3: file of Byte;
        NewID: TID3Rec;
        OldID3: TID3Rec;
        i: Integer;
        sArtist: String;
        sTitle: String;

begin
        sArtist := copy(sFileNameOnly, 1, Pos(sSeparator, sFileNameOnly));
        sTitle := copy(sFileNameOnly, Pos(sSeparator, sFileNameOnly) + Length(sSeparator), Length(sFileNameOnly) - Pos(sSeparator, sFileNameOnly) + 1 - Length(sSeparator));

try

    AssignFile(fMP3, sFullFileName);
    Reset(fMP3);

    try

      Seek(fMP3, FileSize(fMP3) - 128);
      BlockRead(fMP3, OldID3, SizeOf(OldID3));
      if OldID3.Tag = 'TAG' then begin
                                 Seek(fMP3, FileSize(fMP3) - 128);
                                 end
                                 else begin
                                 Seek(fMP3, FileSize(fMP3));
                                 end;

      NewID.Artist := OldID3.Artist;
      NewID.Title := OldID3.Title;
      NewID.Album := OldID3.Album;
      NewID.Year := OldID3.Year;
      NewID.Genre := OldID3.Genre;
      NewID.Comment := OldID3.Comment;
      BlockWrite(fMP3, NewID, SizeOf(NewID));
    finally
    end;

finally
        CloseFile(fMP3);
end;
end;
}
end.
